// Copyright (C) 2019  Wilko Manger
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

import 'dart:typed_data';

import 'package:flutter_cache_manager/flutter_cache_manager.dart';
import 'package:matrix_sdk/matrix_sdk.dart';
import 'package:path/path.dart' as p;
import 'package:path_provider/path_provider.dart';
import 'package:url/url.dart';

class MatrixCacheManager extends BaseCacheManager {
  static const key = 'matrix';

  final Homeserver homeserver;

  MatrixCacheManager(this.homeserver)
      : super(
          key,
          fileFetcher: (url, {headers}) async => _fetch(
            url,
            homeserver,
            headers: headers,
          ),
        );

  Future<String> getPathOf(String url) async {
    final info = await getFileFromCache(url);

    return info?.file?.path;
  }

  @override
  Future<String> getFilePath() async {
    var directory = await getTemporaryDirectory();
    return p.join(directory.path, key);
  }

  static Future<FileFetcherResponse> _fetch(
    String url,
    Homeserver homeserver, {
    Map<String, String> headers,
  }) async {
    final parsedUrl = Url.parse(url);
    var width = parsedUrl.queryParameters['width'];
    var height = parsedUrl.queryParameters['height'];

    Stream<List<int>> stream;
    if (width != null && height != null) {
      stream = await homeserver.downloadThumbnail(
        parsedUrl,
        width: int.parse(width),
        height: int.parse(height),
      );
    } else {
      stream = await homeserver.download(parsedUrl);
    }

    final bytes = List<int>();
    await for (final part in stream) {
      bytes.addAll(part);
    }

    return MatrixFileFetcherResponse(Uint8List.fromList(bytes));
  }
}

class MatrixFileFetcherResponse implements FileFetcherResponse {
  @override
  final Uint8List bodyBytes;

  MatrixFileFetcherResponse(this.bodyBytes);

  @override
  bool hasHeader(String name) => false;

  @override
  String header(String name) => null;

  @override
  get statusCode => 200;
}
